from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
import maya.api.OpenMaya as om
from .. import rigSettings,rigUtil
##モジュールの種類
_MODULETYPE = "setup"

##アトリビュート名リスト
_ATTRS = [
                ##固有アトリビュート
                "targets",
                
                ##予約済みアトリビュート
                
]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "targets":          {"type":"compound","ch":["startJoint","endJoint","aimParent","upParent","upAxis","upPointDistance"],"multi":True},
                    "startJoint":       {"type":"message",  "multi":False,  "default":""},
                    ## aimConstraintで制御されるjoint
                    
                    "endJoint":         {"type":"message",  "multi":False,  "default":""},
                    ## aimtConstraintのtargetの位置参照元. startJoint -> endjointで aimVectorを定義する。
                    
                    "aimParent":        {"type":"message",  "multi":False,  "default":""},
                    ## endJoint位置に作成した aimTargetのparent先
                    
                    "upParent":         {"type":"message",  "multi":False,  "default":""},
                    ## worldUpObject のparent先

                    "upAxis":           {"type":"enum",     "enumList":["x","y","z","-x","-y","-z"],   "default":"y"},
                    "upPointDistance":  {"type":"double",     "multi":False,   "default":0.0, "min":0.0, "max":100},
                    ## startJoint位置にworldUpObjectを作成し、upAxis方向に移動させる数値。　
                    ## 0 の場合にはworldUpType　を　objectrotation　に設定する。
}

##--------------------------------------------------------------------------------------------
def setup(startJoint,endJoint,aimParent,upParent,upAxis,upPointDistance,**kwargs):

    ##startJointの親を取得
    parentJoint = cmds.listRelatives(startJoint,p=True)[0]

    ## start / end / parnt のworldMatrixを取得し、aimVectorを導き出す。
    parentMatrix = om.MMatrix(cmds.getAttr(parentJoint + ".worldMatrix[0]"))
    parentInvMatrix = parentMatrix.inverse()
    startPointMatrix = om.MMatrix(cmds.getAttr(startJoint + ".worldMatrix[0]")) * parentInvMatrix
    endPointMatrix = om.MMatrix(cmds.getAttr(endJoint + ".worldMatrix[0]")) * parentInvMatrix
    startPosition = om.MTransformationMatrix(startPointMatrix).translation(om.MSpace.kObject)
    endPosition = om.MTransformationMatrix(endPointMatrix).translation(om.MSpace.kObject)
    vectorA = om.MVector(startPosition)
    vectorB = om.MVector(endPosition)
    aimVector = vectorB - vectorA
    aimVector = aimVector.normal()
    
    ##endJointの位置に aimTarget となるjointを作成する。
    aimTarget = cmds.createNode("joint",name = startJoint + "_aim")
    rigUtil.snapPosition(endJoint,aimTarget)
    rigUtil.snapRotation(endJoint,aimTarget)
    cmds.parent(aimTarget,aimParent)

    ##startJointの位置に upTarget となるjointを作成する。
    upTarget = cmds.createNode("joint",name = startJoint + "_up")
    rigUtil.snapPosition(startJoint,upTarget)
    rigUtil.snapRotation(startJoint,upTarget)

    if upPointDistance == 0:
        ##upPointDistance が0の場合は　worldUpType を "objectrotation"に設定
        worldUpType = "objectrotation"
    else:
        ##upAxis方向にupTargetを移動
        worldUpType = "object"
        cmds.parent(upTarget,startJoint,a=True)

        if upAxis[0] == "-":
            upPointDistance = upPointDistance * -1.0

        cmds.setAttr(upTarget + ".t"+upAxis[-1],upPointDistance)

    ##upParentの指定が無い場合は、startJointと同階層にparent
    if upParent == "":
        upParent = parentJoint

    cmds.parent(upTarget,upParent,a=True)

    ##aimConstraintを適用
    cmds.aimConstraint(
                            aimTarget,startJoint,
                            aim = aimVector,
                            upVector = rigUtil.stringToVector(upAxis),
                            worldUpType = worldUpType,
                            worldUpObject = upTarget,
                            worldUpVector = rigUtil.stringToVector(upAxis)
                        )

##セットアップ実行用
def main(**kwargs):
    for targets in kwargs["targets"]:
        setup(**targets)

    return