from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
from .. import rigSettings,rigUtil
##モジュールの種類
_MODULETYPE = "setup"

##アトリビュート名リスト
_ATTRS = [
                ##固有アトリビュート
                "rotateJoint",
                "translateJoint",
                "guideMesh",
                "useParamCtrl",
                "useParamPrefix"

                ##予約済みアトリビュート
                
]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "rotateJoint":      {"type":"message",  "multi":False,  "default":""},
                "translateJoint":   {"type":"message",  "multi":False,  "default":""},
                "guideMesh":        {"type":"message",  "multi":False,  "default":""},
                "useParamCtrl":     {"type":"enum",     "enumList":["root","sectionParamCtrl"],   "default":"sectionParamCtrl"},
                "useParamPrefix":   {"type":"enum","enumList":["none","section","section+side","prefix","prefix+side"],"default":"none"},

}

##--------------------------------------------------------------------------------------------

def setupBoxPivot(section,prefix,side,rotateJoint,translateJoint,guideMesh,useParamCtrl,useParamPrefix,sectionSide,**kwargs):
    ##create setupGP
    ##セットアップ用のアレコレを格納する階層を作成
    ##固有名になるように section / prefix / side から生成
    sideString = rigUtil.getSideString(side)
    sectionSetupGP = rigUtil.createSectionSetupGP(section,rigSettings.NODENAME_DICT["setupGP"])
    setupSpace = rigUtil.createSetupSpace(section,prefix,side,None)
    cmds.parent(setupSpace,sectionSetupGP,a=True)

    ##create rotSampleJoint    
    ##セットアップで作成するノード名が固有になるように section / prefix / side から生成
    rotSampleJoint = section + "_" + prefix + "_rotateSample" + sideString + "_jnt"
    cmds.createNode("joint",name = rotSampleJoint)
    rigUtil.snapPosition(rotateJoint,rotSampleJoint)
    rigUtil.snapRotation(rotateJoint,rotSampleJoint)
    cmds.parent(rotSampleJoint,setupSpace,a=True)

    rigUtil.connectAttrZip(rotateJoint,rotSampleJoint,["rotate"],["rotate"])

    ##parent guideMesh
    cmds.parent(guideMesh,setupSpace,a=True)

    ##create botPoint
    ##bbox底面張り付き用joint とりあえず原点・world向き
    BBoxYJoint = section + "_" + prefix + "_BBOXBotY" + sideString + "_jnt"
    cmds.createNode("transform",name = BBoxYJoint)
    cmds.parent(BBoxYJoint,setupSpace,a=True)

    ##bind rotSampleJoint guideMesh
    rigUtil.createSkinCluster(guideMesh,[rotSampleJoint])

    ##connect guideMesh bbox to botPoint translate
    guideMeshShape = cmds.listRelatives(guideMesh,type ="shape")[0]
    
    ##bbox底面位置の変化差分だけを抽出
    translateYValue = cmds.createNode("floatMath") ##sub
    cmds.setAttr(translateYValue + ".operation",1)
    cmds.connectAttr(guideMeshShape + ".boundingBox.boundingBoxMin.boundingBoxMinY",translateYValue + ".floatA")
    cmds.setAttr(translateYValue + ".floatB",cmds.getAttr(guideMeshShape + ".boundingBox.boundingBoxMin.boundingBoxMinY"))

    ##bbox底面位置の変化差分を *weight でon/off
    translateYWeight = cmds.createNode("floatMath") ##multi    
    cmds.setAttr(translateYWeight + ".operation",2)
    cmds.connectAttr(translateYValue + ".outFloat",translateYWeight + ".floatA")
    cmds.setAttr(translateYWeight + ".floatB",0)
    
    ##移動値を反転
    translateYInv = cmds.createNode("floatMath") ##multi
    cmds.setAttr(translateYInv + ".operation",2)
    cmds.connectAttr(translateYWeight + ".outFloat",translateYInv + ".floatA")
    cmds.setAttr(translateYInv + ".floatB",-1)
    cmds.connectAttr(translateYInv + ".outFloat",BBoxYJoint + ".translateY")

    # ##connect botPoint to translateJoint    
    # ##translateJointが　原点・world向き　ではない場合を考慮して、translateJointを複製して出力用ノードを作成
    translatePosJoint = section + "_" + prefix + "_transPos" + sideString + "_jnt"
    cmds.createNode("joint",name = translatePosJoint)
    rigUtil.snapPosition(translateJoint,translatePosJoint)
    rigUtil.snapRotation(translateJoint,translatePosJoint)
    cmds.parent(translatePosJoint,BBoxYJoint,a=True)

    outputTranslateSpace = section + "_" + prefix + "_transSpace" + sideString + "_jnt"
    translateParentJoint = cmds.listRelatives(translateJoint,p=True)[0]
    cmds.createNode("joint",name = outputTranslateSpace)
    rigUtil.snapPosition(translateParentJoint,outputTranslateSpace)
    rigUtil.snapRotation(translateParentJoint,outputTranslateSpace)
    cmds.parent(outputTranslateSpace,setupSpace,a=True)

    outputTranslateJoint = section + "_" + prefix + "_trans" + sideString + "_jnt"
    cmds.createNode("joint",name = outputTranslateJoint)
    rigUtil.snapPosition(translateJoint,outputTranslateJoint)
    rigUtil.snapRotation(translateJoint,outputTranslateJoint)    
    cmds.parent(outputTranslateJoint,outputTranslateSpace,a=True)

    cmds.parentConstraint(translatePosJoint,outputTranslateJoint,mo=False)

    # ##add switchAttr
    paramCtrlnode = rigSettings.NODENAME_DICT["rootGP"]
    
    if useParamCtrl == "sectionParamCtrl":
        sectionSideString = rigUtil.getSideString(sectionSide)        
        paramCtrlnode = "PM_" + section + sectionSideString + rigSettings.SUFFIX_DICT["ctrl"]
        
        ##パラメーター名が他モジュールと被らないように section / side / prefix から生成
        switchAttrName = rigUtil.geneParamName(useParamPrefix,"bboxPosWeight",section,side,prefix)

    if cmds.attributeQuery(switchAttrName, node = paramCtrlnode, exists =True) == False:
        cmds.addAttr(paramCtrlnode, ln = switchAttrName, at = "double", dv = 0, k =True, min = 0, max =1)
    
    cmds.connectAttr(paramCtrlnode + "."+switchAttrName ,translateYWeight + ".floatB",f=True)
    
    rigUtil.connectAttrZip(outputTranslateJoint,translateJoint,["t"],["t"])
    
##セットアップ実行用
def main(**kwargs):
    setupBoxPivot(**kwargs)
    return