from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
from .. import rigSettings,rigUtil
##モジュールの種類
_MODULETYPE = "setup"

##アトリビュート名リスト
_ATTRS = [
                ##固有アトリビュート
                "guideJoint",
                
                "useTranslate",
                "useRotate",
                "useScale",
                
                "guideMesh",

                "outputs",

                "useParamCtrl",
                "useParamPrefix"
                
                ##予約済みアトリビュート
                
]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "guideJoint":         {"type":"message",  "multi":False,  "default":""},
                "guideMesh":          {"type":"message",  "multi":False,  "default":""},

                "useTranslate":       {"type":"bool",  "multi":False,  "default":False},
                "useRotate":          {"type":"bool",  "multi":False,  "default":True},
                "useScale":           {"type":"bool",  "multi":False,  "default":False},

                "outputs":              {"type":"compound",  "multi":True,  "ch":["outputTarget","outputPosition","outputX","outputY","outputZ","useOffset","connectWeight"]},
                    "outputTarget":         {"type":"message",  "multi":False,  "default":""},
                    "outputPosition":       {"type":"enum",     "enumList":["Min","Max"],   "default":"Min"},

                    "outputX":              {"type":"enum",  "enumList":["+","-","none"],   "default":"+"},
                    "outputY":              {"type":"enum",  "enumList":["+","-","none"],   "default":"+"},
                    "outputZ":              {"type":"enum",  "enumList":["+","-","none"],   "default":"+"},

                    "useOffset":            {"type":"bool",  "multi":False,  "default":True},
                    "connectWeight":        {"type":"bool",  "multi":False,  "default":True},

                "useParamCtrl":     {"type":"enum",     "enumList":["root","sectionParamCtrl"],   "default":"sectionParamCtrl"},
                "useParamPrefix":   {"type":"enum","enumList":["none","section","section+side","prefix","prefix+side"],"default":"none"},
}

##--------------------------------------------------------------------------------------------

def setupBBoxPositions(section,sectionSide,prefix,side,guideJoint,guideMesh,outputs,useParamCtrl,useParamPrefix,useTranslate,useRotate,useScale,**kwargs):
    
    ##create setupGP
    ##セットアップ用のアレコレを格納する階層を作成
    ##固有名になるように section / prefix / side から生成
    sideString = rigUtil.getSideString(side)
    sectionSetupGP = rigUtil.createSectionSetupGP(section,rigSettings.NODENAME_DICT["setupGP"])
    
    setupSpace = rigUtil.createSetupSpace(section,prefix,side,None)
    cmds.parent(setupSpace,sectionSetupGP,a=True)

    ##setup guide---------------------------------------------------------
    ##create guideSampleJoint    
    ##セットアップで作成するノード名が固有になるように section / prefix / side から生成
    guideSampleJoint = section + "_" + prefix + "_guide" + sideString + "_jnt"
    cmds.createNode("joint",name = guideSampleJoint)
    rigUtil.snapPosition(guideJoint,guideSampleJoint)
    rigUtil.snapRotation(guideJoint,guideSampleJoint)

    guideParentJoint = cmds.listRelatives(guideJoint,p=True)[0]
    setupGuideSpace = cmds.createNode("joint",name = section + "_" + prefix + "_guide" + sideString + "_space")
    rigUtil.snapPosition(guideParentJoint,setupGuideSpace)
    rigUtil.snapRotation(guideParentJoint,setupGuideSpace)

    cmds.parent(setupGuideSpace,setupSpace,a=True)
    cmds.parent(guideSampleJoint,setupGuideSpace,a=True)

    ##connect guide attrs
    
    if useTranslate:
        rigUtil.connectAttrZip(guideJoint,guideSampleJoint,["t"],["t"])
    if useRotate:
        rigUtil.connectAttrZip(guideJoint,guideSampleJoint,["r"],["r"])
    if useScale:
        rigUtil.connectAttrZip(guideJoint,guideSampleJoint,["s"],["s"])

    ##parent guideMesh
    cmds.parent(guideMesh,setupSpace,a=True)

    ##bind guideSampleJoint guideMesh
    rigUtil.createSkinCluster(guideMesh,[guideSampleJoint])

    ##setup bbox positions
    ##出来れば任意の空間準拠でのbboxSizeを取得したいがー　今回はworld空間準拠で
    guideMeshShape = cmds.listRelatives(guideMesh,type ="shape")[0]    
    
    for outputSetting in outputs:

        outputTarget = outputSetting["outputTarget"]
        outputPosition = outputSetting["outputPosition"] ##Min or Max
        
        outputTargetParent = cmds.listRelatives(outputTarget,p=True)[0]

        BBOXPosNode = outputTarget + "_sourcePos"
        inputPosNode = outputTarget + "_inputPos"
        outputPosSpace = outputTarget + "_outputSpace"
        outputPosNode = outputTarget + "_outputPos"

        cmds.createNode("transform",name = BBOXPosNode)
        cmds.parent(BBOXPosNode,setupSpace,a=True)
        
        ##変化差分だけを抽出(offset不要の場合は0)
        translateValue = cmds.createNode("plusMinusAverage") 
        cmds.setAttr(translateValue + ".operation",2) ##sub
        
        cmds.connectAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition+"X",translateValue + ".input3D[0].input3Dx")
        cmds.connectAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition+"Y",translateValue + ".input3D[0].input3Dy")
        cmds.connectAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition+"Z",translateValue + ".input3D[0].input3Dz")
        
        if outputSetting["useOffset"]:
            cmds.setAttr(translateValue + ".input3D[1].input3Dx",cmds.getAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition + "X"))
            cmds.setAttr(translateValue + ".input3D[1].input3Dy",cmds.getAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition + "Y"))
            cmds.setAttr(translateValue + ".input3D[1].input3Dz",cmds.getAttr(guideMeshShape + ".boundingBox.boundingBox"+outputPosition+".boundingBox"+outputPosition + "Z"))

        ##変化差分を *weight でon/off
        translateWeight = cmds.createNode("multiplyDivide") 
        cmds.setAttr(translateWeight + ".operation",1) ##multi
        cmds.connectAttr(translateValue + ".output3Dx",translateWeight + ".input1X")
        cmds.connectAttr(translateValue + ".output3Dy",translateWeight + ".input1Y")
        cmds.connectAttr(translateValue + ".output3Dz",translateWeight + ".input1Z")

        cmds.setAttr(translateWeight + ".input2X",1)
        cmds.setAttr(translateWeight + ".input2Y",1)
        cmds.setAttr(translateWeight + ".input2Z",1)

        if outputSetting["connectWeight"]:
            # ##add switchAttr
            paramCtrlnode = rigSettings.NODENAME_DICT["rootGP"]
            
            if useParamCtrl == "sectionParamCtrl":
                sectionSideString = rigUtil.getSideString(sectionSide)        
                paramCtrlnode = "PM_" + section + sectionSideString + rigSettings.SUFFIX_DICT["ctrl"]
                
            ##パラメーター名が他モジュールと被らないように section / side / prefix から生成
            switchAttrName = rigUtil.geneParamName(useParamPrefix,"bboxPosWeight",section,side,prefix)

            if cmds.attributeQuery(switchAttrName, node = paramCtrlnode, exists =True) == False:
                cmds.addAttr(paramCtrlnode, ln = switchAttrName, at = "double", dv = 1, k =True, min = 0, max =1)
            
            cmds.connectAttr(paramCtrlnode + "."+switchAttrName ,translateWeight + ".input2X",f=True)
            cmds.connectAttr(paramCtrlnode + "."+switchAttrName ,translateWeight + ".input2Y",f=True)
            cmds.connectAttr(paramCtrlnode + "."+switchAttrName ,translateWeight + ".input2Z",f=True)

        ##出力軸を設定
        ##"outputX","outputY","outputZ"
        translateInv = cmds.createNode("multiplyDivide")  ##multi
        cmds.setAttr(translateInv + ".operation",1)
        
        for axis in ["X","Y","Z"]:
            cmds.connectAttr(translateWeight + ".output"+axis,translateInv + ".input1"+axis)            
            cmds.setAttr(translateInv + ".input2"+axis, 1)

            if outputSetting["output"+ axis] == "-":
                cmds.setAttr(translateInv + ".input2"+axis, -1)
            
            if outputSetting["output"+ axis] != "none":
                cmds.connectAttr(translateInv + ".output"+axis, BBOXPosNode + ".translate"+axis)

        cmds.createNode("joint",name = inputPosNode)
        rigUtil.snapPosition(outputTarget,inputPosNode)
        rigUtil.snapRotation(outputTarget,inputPosNode)
        cmds.parent(inputPosNode,BBOXPosNode,a=True)
        
        cmds.createNode("joint",name = outputPosSpace)
        rigUtil.snapPosition(outputTargetParent,outputPosSpace)
        rigUtil.snapRotation(outputTargetParent,outputPosSpace)
        cmds.parent(outputPosSpace,setupSpace,a=True)

        cmds.createNode("joint",name = outputPosNode)
        rigUtil.snapPosition(outputTarget,outputPosNode)
        rigUtil.snapRotation(outputTarget,outputPosNode)
        cmds.parent(outputPosNode,outputPosSpace,a=True)

        cmds.parentConstraint(BBOXPosNode,outputPosNode,mo = False)
        rigUtil.connectAttrZip(outputPosNode,outputTarget,["t"],["t"])
    
##セットアップ実行用
def main(**kwargs):    
    setupBBoxPositions(**kwargs)
    return
