from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
from .. import rigUtil,rigSettings
##モジュールの種類
_MODULETYPE = "deform"

##アトリビュート名リスト
_ATTRS = [
            ##固有アトリビュート
            "sDivisions",
            "tDivisions",
            "uDivisions",
            
            "sLocalInfluence",
            "tLocalInfluence",
            "uLocalInfluence",
            
            "outsideLattice",

            "latticeGuide",
            "latticeParent",
            "baseParent",

            "targets"
            ]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "sDivisions":           {"type":"long","default":2,"max":9999,"min":2},
                "tDivisions":           {"type":"long","default":2,"max":9999,"min":2},
                "uDivisions":           {"type":"long","default":2,"max":9999,"min":2},

                "sLocalInfluence":      {"type":"long","default":2,"max":9999,"min":2},
                "tLocalInfluence":      {"type":"long","default":2,"max":9999,"min":2},
                "uLocalInfluence":      {"type":"long","default":2,"max":9999,"min":2},

                "outsideLattice":       {"type":"enum","enumList":["inside","all","fallOff"],"default":"all"},

                "latticeGuide":         {"type":"message","default":"","multi":False},

                "latticeParent":        {"type":"string","default":"","multi":False},
                "targetPrefix":         {"type":"string","default":"","multi":False},
                "baseParent":           {"type":"string","default":"","multi":False},

                "targets":              {"type":"string","default":"","multi":True},

                }

##------------------------------------------------------------------------------------------------
def createLattice(targets,latticeName,outsideLattice,divisions,localDiv,position = None,rotation = None,scale = None):
    ffdNode,lattice,base = cmds.lattice(
                                        targets,
                                        name = latticeName + "Ffd",
                                        divisions = divisions,
                                        ldivisions = localDiv,
                                        commonParent = False,
                                        objectCentered = True,
                                        outsideLattice = outsideLattice
                                        )

    if position != None:
        cmds.setAttr(lattice + ".t",*position)
        cmds.setAttr(base + ".t",*position)

    if rotation != None:
        cmds.setAttr(lattice + ".r",*rotation)
        cmds.setAttr(base + ".r",*rotation)

    if scale != None:
        cmds.setAttr(lattice + ".s",*scale)
        cmds.setAttr(base + ".s",*scale)

    return ffdNode,lattice,base

##---------------------------------------------------------------------------------------------
def setup(section,prefix,side,targets,divisions,localDiv,position,rotation,scale,outsideLattice,latticeParent,baseParent,**kwargs):

    deformerPrefix = rigUtil.addPrefix(section, prefix)
    sideString = rigUtil.getSideString(side)
    
    outsideOpt = {"inside":0,"all":1,"fallOff":2}
    
    deformTargets = []
    for target in targets:
        if cmds.objExists(target):
            deformTargets.append(target)
        
        else:
            ##ターゲットが見つからない場合はエラーにしてreturnするのも場合によっては有
            print("do not exists " + target)

    ##  deformerPrefix + sideString + _Ffd  をノード名にする
    ffdNode,lattice,base = createLattice(deformTargets,deformerPrefix + sideString ,outsideOpt[outsideLattice],divisions,localDiv,position,rotation,scale)

    ##指定が無い場合は setup_gpに格納する
    if latticeParent == "":
        latticeParent = rigSettings.NODENAME_DICT["setupGP"]

    if cmds.objExists(latticeParent) == False:
        latticeParent = rigSettings.NODENAME_DICT["setupGP"]

    ## root_jnt 内を指定していた場合は proxyを作成してそちらに変更
    latticeParentHie = rigUtil.getCurHierarchy(latticeParent)
    if rigSettings.NODENAME_DICT["rootJNT"] in latticeParentHie:
        latticeParent = rigUtil.createParentProxy(latticeParent,rigSettings.NODENAME_DICT["ctrlWorldSpace"])

    if baseParent == "":
        baseParent = latticeParent

    if cmds.objExists(baseParent) == False:
        baseParent = latticeParent

    ## root_jnt 内を指定していた場合は proxyを作成してそちらに変更
    baseParentHie = rigUtil.getCurHierarchy(baseParent)
    if rigSettings.NODENAME_DICT["rootJNT"] in baseParentHie:
        baseParent = rigUtil.createParentProxy(baseParent,rigSettings.NODENAME_DICT["ctrlWorldSpace"])

    cmds.parent(lattice,latticeParent,a=True)
    cmds.parent(base,baseParent,a=True)

    cmds.setAttr(lattice + ".v", False)
    cmds.setAttr(base + ".v", False)

def main(**kwargs):

    setupParam = {
                    "section":          kwargs["section"],
                    "prefix":           kwargs["prefix"],
                    "side":             kwargs["side"],
                    
                    "targets":          kwargs["targets"],
                    "outsideLattice":   kwargs["outsideLattice"],
                    "latticeParent":    kwargs["latticeParent"],
                    "baseParent":       kwargs["baseParent"],
                    
                    "divisions":    [kwargs["sDivisions"],kwargs["tDivisions"],kwargs["uDivisions"]],
                    "localDiv":     [kwargs["sLocalInfluence"],kwargs["tLocalInfluence"],kwargs["uLocalInfluence"]]
                }

    if kwargs["latticeGuide"] != "":
        setupParam["position"] = cmds.getAttr(kwargs["latticeGuide"] + ".t")[0]
        setupParam["rotation"] = cmds.getAttr(kwargs["latticeGuide"] + ".r")[0]
        setupParam["scale"] = cmds.getAttr(kwargs["latticeGuide"] + ".s")[0]
    else:
        setupParam["position"] = None
        setupParam["rotation"] = None
        setupParam["scale"] = None
            
    setup(**setupParam)

