from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
from .. import rigSettings,rigUtil
##モジュールの種類
_MODULETYPE = "setup"

##アトリビュート名リスト
_ATTRS = [
                ##固有アトリビュート
                "startJoint",
                "endJoint",
                "parentSpace",

                "outputs",

                "useParamCtrl",
                "useParamPrefix",

                ##予約済みアトリビュート
                
]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "startJoint":       {"type":"message",  "multi":False,  "default":""},
                "endJoint":         {"type":"message",  "multi":False,  "default":""},
                "parentSpace":         {"type":"message",  "multi":False,  "default":""},

                "outputs":              {"type":"compound",  "multi":True,  "ch":["targetJoint","directionAxis","useWidthScale","useWidthScaleLimit","connectWeight"]},
                    "targetJoint":        {"type":"message",  "multi":False,  "default":""},
                    "directionAxis":      {"type":"enum",     "enumList":["x","y","z"],   "default":"x"},
                    "useWidthScale":        {"type":"bool",  "multi":False,  "default":False},
                    "useWidthScaleLimit":   {"type":"bool",  "multi":False,  "default":False},
                    "connectWeight":        {"type":"bool",  "multi":False,  "default":False},

                "useParamCtrl":     {"type":"enum",     "enumList":["root","sectionParamCtrl"],   "default":"sectionParamCtrl"},
                "useParamPrefix":   {"type":"enum","enumList":["none","section","section+side","prefix","prefix+side"],"default":"none"},
}

##--------------------------------------------------------------------------------------------

def setup(section,sectionSide,prefix,side,startJoint,endJoint,parentSpace,useParamCtrl,useParamPrefix,outputs,**kwargs):
    
    ##parentSpace指定が無い場合は、startJointの親をparentSapceとして採用
    if parentSpace == None or parentSpace == "":
        parentSpace = cmds.listRelatives(startJoint,p=True)[0]

    startMultmatrix = cmds.createNode("multMatrix")
    cmds.connectAttr(startJoint + ".worldMatrix[0]",startMultmatrix + ".matrixIn[0]")    
    cmds.connectAttr(parentSpace + ".worldInverseMatrix[0]",startMultmatrix + ".matrixIn[1]")

    endMultmatrix = cmds.createNode("multMatrix")
    cmds.connectAttr(endJoint + ".worldMatrix[0]",endMultmatrix + ".matrixIn[0]")    
    cmds.connectAttr(parentSpace + ".worldInverseMatrix[0]",endMultmatrix + ".matrixIn[1]")

    ##setup squash
    distance = cmds.createNode("distanceBetween")
    cmds.connectAttr(startMultmatrix + ".matrixSum", distance + ".inMatrix1")
    cmds.connectAttr(endMultmatrix + ".matrixSum", distance + ".inMatrix2")
    
    distToLeghthScale = cmds.createNode("floatMath")
    cmds.setAttr(distToLeghthScale + ".operation",3)
    cmds.connectAttr(distance + ".distance", distToLeghthScale + ".floatA")
    cmds.setAttr(distToLeghthScale + ".floatB", cmds.getAttr(distance + ".distance"))

    ## 太さ方向スケールのwight(全体)
    switchAttrName = rigUtil.geneParamName(useParamPrefix,"widthScaleWeight",section,side,prefix)
    paramCtrlnode = rigSettings.NODENAME_DICT["rootGP"]

    if useParamCtrl == "sectionParamCtrl":
        sectionSideString = rigUtil.getSideString(sectionSide)        
        paramCtrlnode = "PM_" + section + sectionSideString + rigSettings.SUFFIX_DICT["ctrl"]

    for output in outputs:
        #"targetJoint","directionAxis","useWidthScale","connectWeight"
        targetJoint = output["targetJoint"]
        jointNamePrefix = targetJoint.replace(rigSettings.SUFFIX_DICT["jnt"],"")

        rigUtil.connectAttrMerge(distToLeghthScale + ".outFloat",targetJoint , "s" + output["directionAxis"])

        if output["useWidthScale"]:
            
            scaleToWidth = cmds.createNode("floatMath")
            cmds.setAttr(scaleToWidth + ".operation",3)
            cmds.setAttr(scaleToWidth +".floatA",1.0)
            cmds.connectAttr(distToLeghthScale + ".outFloat", scaleToWidth + ".floatB")

            blendAttr = cmds.createNode("blendTwoAttr")
            cmds.setAttr(blendAttr + ".input[0]",1.0)

            if output["useWidthScaleLimit"]:
            ## 太さのスケール値リミット
                limitMaxName = rigUtil.geneParamName(useParamPrefix,"widthScaleMax",section,side,prefix)
                if cmds.attributeQuery(limitMaxName, node = paramCtrlnode, exists =True) == False:
                    cmds.addAttr(paramCtrlnode, ln = limitMaxName, at = "double", dv = 10, k =True, min = 0.001)

                limitMinName = rigUtil.geneParamName(useParamPrefix,"widthScaleMin",section,side,prefix)
                if cmds.attributeQuery(limitMinName, node = paramCtrlnode, exists =True) == False:
                    cmds.addAttr(paramCtrlnode, ln = limitMinName, at = "double", dv = 0.1, k =True, min = 0.001)

                clamp = cmds.createNode("clamp")
                cmds.connectAttr(scaleToWidth + ".outFloat",clamp + ".inputR")
                cmds.connectAttr(paramCtrlnode + "."+limitMaxName,clamp + ".maxR")
                cmds.connectAttr(paramCtrlnode + "."+limitMinName,clamp + ".minR")

                cmds.connectAttr(clamp + ".outputR", blendAttr + ".input[1]")

            if output["connectWeight"]:
                if cmds.attributeQuery(switchAttrName, node = paramCtrlnode, exists =True) == False:
                    cmds.addAttr(paramCtrlnode, ln = switchAttrName, at = "double", dv = 1.0, k =True, min = 0, max =1)

                ## 太さ方向スケールのwight(個別)
                partWeightAttrName = rigUtil.geneParamName(useParamPrefix, jointNamePrefix + "widthScaleWeight",section,"",prefix)
                if cmds.attributeQuery(partWeightAttrName, node = paramCtrlnode, exists =True) == False:
                    cmds.addAttr(paramCtrlnode, ln = partWeightAttrName, at = "double", dv = 1.0, k =True, min = 0)

                multiWeight = cmds.createNode("floatMath")
                cmds.setAttr(multiWeight + ".operation",2)
                cmds.connectAttr(paramCtrlnode + "."+switchAttrName, multiWeight + ".floatA")
                cmds.connectAttr(paramCtrlnode + "."+partWeightAttrName, multiWeight + ".floatB")
                cmds.connectAttr(multiWeight + ".outFloat",blendAttr + ".attributesBlender")


            for axis in ["x","y","z"]:
                if axis != output["directionAxis"]:
                    rigUtil.connectAttrMerge(blendAttr + ".output",targetJoint , "s" + axis)
        
    
##セットアップ実行用
def main(**kwargs):
    setup(**kwargs)
    return
