from __future__ import (absolute_import, division,print_function)
import maya.cmds as cmds
from .. import rigSettings,rigUtil
##モジュールの種類
_MODULETYPE = "ctrl"

##アトリビュート名リスト
_ATTRS = [
                ##固有アトリビュート
                "joints",

                "useTranslateX",
                "useTranslateY",
                "useTranslateZ",
                "useRotateX",
                "useRotateY",
                "useRotateZ",
                "useScaleX",
                "useScaleY",
                "useScaleZ",

                ##予約済みアトリビュート
                "ctrlShape",
                "ctrlColor"
]

##アトリビュートのタイプ　デフォルト値などオプションを定義
_ATTR_DICT = {
                "joints":  {"type":"message",  "multi":True,  "default":""},
                
                "useTranslateX": {"type":"bool",  "multi":True,   "default":True},
                "useTranslateY": {"type":"bool",  "multi":True,   "default":True},
                "useTranslateZ": {"type":"bool",  "multi":True,   "default":True},
                
                "useRotateX": {"type":"bool",  "multi":True,   "default":False},
                "useRotateY": {"type":"bool",  "multi":True,   "default":False},
                "useRotateZ": {"type":"bool",  "multi":True,   "default":False},

                "useScaleX": {"type":"bool",  "multi":True,   "default":False},
                "useScaleY": {"type":"bool",  "multi":True,   "default":False},
                "useScaleZ": {"type":"bool",  "multi":True,   "default":False}
}

##--------------------------------------------------------------------------------------------

def setupPointCtrl(section,joint,ctrlShape,ctrlColor,ctrlSize,lockAttrs):
    ##get parentJoint
    parentJoint = cmds.listRelatives(joint,p=True)[0]
    parentProxy = rigSettings.NODENAME_DICT["ctrlWorldSpace"]

    if parentJoint != rigSettings.NODENAME_DICT["rootJNT"]:
        ##create parentProxy
        ##connect jointParent to parentProxy
        parentProxy = rigUtil.createParentProxy(parentJoint,rigSettings.NODENAME_DICT["ctrlWorldSpace"],attrs = ["t","r","s"])

    ##create ctrlNode    
    ctrlNode = rigUtil.createCtrlNode(
                                        joint.replace(rigSettings.SUFFIX_DICT["jnt"],rigSettings.SUFFIX_DICT["ctrl"]),
                                        ctrlShape,
                                        "joint", 
                                        size = ctrlSize,
                                        position = joint,
                                        rotation =joint
                                        )

    cmds.parent(ctrlNode,parentProxy,a=True)
    
    ##add ctrlOffset
    rigUtil.insertOffsetNode("joint",ctrlNode,"_offset")

    ##connect ctrl to joint
    rigUtil.matrixConstraint(ctrlNode,joint,parentProxy,parentJoint,attrs = ["t","r"],offset = False)
    rigUtil.connectAttrZip(ctrlNode,joint,["scaleX","scaleY","scaleZ"],["scaleX","scaleY","scaleZ"],True)
    
    ##不要なアトリビュートをロック    
    rigUtil.setAttrState([ctrlNode],lockAttrs,True,True,False)

    ##コントローラーの色を設定
    rigUtil.setWireFrameColor([ctrlNode],ctrlColor, 0.8)
    
    ##コントローラーを setsへ登録
    rigUtil.addToSet(rigSettings.NODENAME_DICT["allCtrlSet"] + "|" + section + "Ctrl" + rigSettings.SUFFIX_DICT["set"],[ctrlNode])


##セットアップ実行用
def main(**kwargs):
    
    lockAttrs = ["v"]

    useAttrKeys = [
                "useTranslateX",
                "useTranslateY",
                "useTranslateZ",
                "useRotateX",
                "useRotateY",
                "useRotateZ",
                "useScaleX",
                "useScaleY",
                "useScaleZ",
    ]

    for useAttrKey in useAttrKeys:
        if kwargs[useAttrKey] ==False:
            attr = useAttrKey.replace("use","")
            attr = attr[0].lower() + attr[1:]
            lockAttrs.append(attr)

    for joint in kwargs["joints"]:
        setupPointCtrl(kwargs["section"],joint,kwargs["ctrlShape"],kwargs["ctrlColor"],kwargs["ctrlScale"],lockAttrs)

    return
